@@ -47,11 +47,13 @@ struct __MCF_gthr_thread_record
  * FIXME: At the moment (2024-03-14) GCC does not support SEH on i686.  */
 __MCF_GTHR_AUX_INLINE
 void
-__MCF_gthr_call_once_seh(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, void* __arg) __MCF_MAY_THROW;
+__MCF_gthr_call_once_seh(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, void* __arg)
+  __MCF_MAY_THROW;
 
 __MCF_GTHR_AUX_IMPORT
 void
-__MCF_gthr_call_once_seh_take_over(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, void* __arg) __MCF_MAY_THROW;
+__MCF_gthr_call_once_seh_take_over(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, void* __arg)
+  __MCF_MAY_THROW;
 
 /* This is a type-generic macro. `__init_proc` may be of any calling convention,
  * and may take an arbitrary pointer.  */
@@ -62,88 +64,107 @@ __MCF_gthr_call_once_seh_take_over(_MCF_once* __once, __MCF_cxa_dtor_any_ __init
  * number of milliseconds since the Unix epoch, with boundary checking.  */
 __MCF_GTHR_AUX_IMPORT __MCF_FN_PURE
 int64_t
-__MCF_gthr_timeout_from_timespec(const __MCF_timespec* __abs_time) __MCF_noexcept;
+__MCF_gthr_timeout_from_timespec(const __MCF_timespec* __abs_time)
+  __MCF_noexcept;
 
 /* These are auxiliary functions to implement recursive mutexes, and are not
  * to be called directly.  */
 __MCF_GTHR_AUX_INLINE
 void
-__MCF_gthr_rc_mutex_init(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept;
+__MCF_gthr_rc_mutex_init(__MCF_gthr_rc_mutex* __rmtx)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_INLINE
 int
-__MCF_gthr_rc_mutex_recurse(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept;
+__MCF_gthr_rc_mutex_recurse(__MCF_gthr_rc_mutex* __rmtx)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_INLINE
 int
-__MCF_gthr_rc_mutex_wait(__MCF_gthr_rc_mutex* __rmtx, const int64_t* __timeout_opt) __MCF_noexcept;
+__MCF_gthr_rc_mutex_wait(__MCF_gthr_rc_mutex* __rmtx, const int64_t* __timeout_opt)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_INLINE
 void
-__MCF_gthr_rc_mutex_release(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept;
+__MCF_gthr_rc_mutex_release(__MCF_gthr_rc_mutex* __rmtx)
+  __MCF_noexcept;
 
 /* These are auxiliary functions for condition variables. The argument is a
  * pointer to a plain `_MCF_mutex`.  */
 __MCF_GTHR_AUX_IMPORT
 intptr_t
-__MCF_gthr_mutex_unlock_callback(intptr_t __arg) __MCF_noexcept;
+__MCF_gthr_mutex_unlock_callback(intptr_t __arg)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 void
-__MCF_gthr_mutex_relock_callback(intptr_t __arg, intptr_t __unlocked) __MCF_noexcept;
+__MCF_gthr_mutex_relock_callback(intptr_t __arg, intptr_t __unlocked)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 int
-__MCF_gthr_cond_mutex_wait(_MCF_cond* __cond, _MCF_mutex* __mtx, int64_t* __timeout_opt) __MCF_noexcept;
+__MCF_gthr_cond_mutex_wait(_MCF_cond* __cond, _MCF_mutex* __mtx, int64_t* __timeout_opt)
+  __MCF_noexcept;
 
 /* These are auxiliary functions for condition variables. The argument is a
  * pointer to a `_MCF_shared_mutex`.  */
 __MCF_GTHR_AUX_IMPORT
 intptr_t
-__MCF_gthr_shared_mutex_unlock_callback(intptr_t __arg) __MCF_noexcept;
+__MCF_gthr_shared_mutex_unlock_callback(intptr_t __arg)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 void
-__MCF_gthr_shared_mutex_relock_shared_callback(intptr_t __arg, intptr_t __unlocked) __MCF_noexcept;
+__MCF_gthr_shared_mutex_relock_shared_callback(intptr_t __arg, intptr_t __unlocked)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 void
-__MCF_gthr_shared_mutex_relock_exclusive_callback(intptr_t __arg, intptr_t __unlocked) __MCF_noexcept;
+__MCF_gthr_shared_mutex_relock_exclusive_callback(intptr_t __arg, intptr_t __unlocked)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 int
-__MCF_gthr_cond_shared_mutex_wait_shared(_MCF_cond* __cond, _MCF_shared_mutex* __smtx, int64_t* __timeout_opt) __MCF_noexcept;
+__MCF_gthr_cond_shared_mutex_wait_shared(_MCF_cond* __cond, _MCF_shared_mutex* __smtx, int64_t* __timeout_opt)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 int
-__MCF_gthr_cond_shared_mutex_wait_exclusive(_MCF_cond* __cond, _MCF_shared_mutex* __smtx, int64_t* __timeout_opt) __MCF_noexcept;
+__MCF_gthr_cond_shared_mutex_wait_exclusive(_MCF_cond* __cond, _MCF_shared_mutex* __smtx, int64_t* __timeout_opt)
+  __MCF_noexcept;
 
 /* These are auxiliary functions for condition variables. The argument is a
  * pointer to a `__MCF_gthr_rc_mutex`.  */
 __MCF_GTHR_AUX_IMPORT
 intptr_t
-__MCF_gthr_recursive_mutex_unlock_callback(intptr_t __arg) __MCF_noexcept;
+__MCF_gthr_recursive_mutex_unlock_callback(intptr_t __arg)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 void
-__MCF_gthr_recursive_mutex_relock_callback(intptr_t __arg, intptr_t __unlocked) __MCF_noexcept;
+__MCF_gthr_recursive_mutex_relock_callback(intptr_t __arg, intptr_t __unlocked)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 int
-__MCF_gthr_cond_recursive_mutex_wait(_MCF_cond* __cond, __MCF_gthr_rc_mutex* __rmtx, int64_t* __timeout_opt) __MCF_noexcept;
+__MCF_gthr_cond_recursive_mutex_wait(_MCF_cond* __cond, __MCF_gthr_rc_mutex* __rmtx, int64_t* __timeout_opt)
+  __MCF_noexcept;
 
 /* These are auxiliary functions for threads.  */
 __MCF_GTHR_AUX_IMPORT
 _MCF_thread*
-__MCF_gthr_thread_create_v3(__MCF_gthr_thread_procedure* __proc, void* __arg) __MCF_noexcept;
+__MCF_gthr_thread_create_v3(__MCF_gthr_thread_procedure* __proc, void* __arg)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT
 void
-__MCF_gthr_thread_join_v3(_MCF_thread* __thrd, void** __resp_opt) __MCF_noexcept;
+__MCF_gthr_thread_join_v3(_MCF_thread* __thrd, void** __resp_opt)
+  __MCF_noexcept;
 
 __MCF_GTHR_AUX_IMPORT __MCF_NEVER_RETURN
 void
-__MCF_gthr_thread_exit_v3(void* __resp) __MCF_noexcept;
+__MCF_gthr_thread_exit_v3(void* __resp)
+  __MCF_noexcept;
 
 /* Define inline functions after all declarations.
  * We would like to keep them away from declarations for conciseness, which also
@@ -152,7 +173,8 @@ __MCF_gthr_thread_exit_v3(void* __resp) __MCF_noexcept;
  * this file.  */
 __MCF_GTHR_AUX_INLINE
 void
-__MCF_gthr_call_once_seh(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, void* __arg) __MCF_MAY_THROW
+__MCF_gthr_call_once_seh(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, void* __arg)
+  __MCF_MAY_THROW
   {
     int __err = _MCF_once_wait(__once, __MCF_nullptr);
     if(__err == 0)
@@ -164,7 +186,8 @@ __MCF_gthr_call_once_seh(_MCF_once* __once, __MCF_cxa_dtor_any_ __init_proc, voi
 
 __MCF_GTHR_AUX_INLINE
 void
-__MCF_gthr_rc_mutex_init(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept
+__MCF_gthr_rc_mutex_init(__MCF_gthr_rc_mutex* __rmtx)
+  __MCF_noexcept
   {
     __rmtx->__owner[0] = 0;
     __rmtx->__depth = 0;
@@ -173,7 +196,8 @@ __MCF_gthr_rc_mutex_init(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept
 
 __MCF_GTHR_AUX_INLINE
 int
-__MCF_gthr_rc_mutex_recurse(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept
+__MCF_gthr_rc_mutex_recurse(__MCF_gthr_rc_mutex* __rmtx)
+  __MCF_noexcept
   {
     /* Check whether the mutex has already been owned.  */
     if(_MCF_atomic_load_32_rlx(__rmtx->__owner) != (int32_t) _MCF_thread_self_tid())
@@ -187,7 +211,8 @@ __MCF_gthr_rc_mutex_recurse(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept
 
 __MCF_GTHR_AUX_INLINE
 int
-__MCF_gthr_rc_mutex_wait(__MCF_gthr_rc_mutex* __rmtx, const int64_t* __timeout_opt) __MCF_noexcept
+__MCF_gthr_rc_mutex_wait(__MCF_gthr_rc_mutex* __rmtx, const int64_t* __timeout_opt)
+  __MCF_noexcept
   {
     /* Attempt to take ownership.  */
     int __err = _MCF_mutex_lock(__rmtx->__mutex, __timeout_opt);
@@ -204,7 +229,8 @@ __MCF_gthr_rc_mutex_wait(__MCF_gthr_rc_mutex* __rmtx, const int64_t* __timeout_o
 
 __MCF_GTHR_AUX_INLINE
 void
-__MCF_gthr_rc_mutex_release(__MCF_gthr_rc_mutex* __rmtx) __MCF_noexcept
+__MCF_gthr_rc_mutex_release(__MCF_gthr_rc_mutex* __rmtx)
+  __MCF_noexcept
   {
     /* Reduce a level of recursion.  */
     __MCF_ASSERT(__rmtx->__depth > 0);
